/*
 * Copyright 2017 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.kappich.pat.gnd.
 * 
 * de.kappich.pat.gnd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.kappich.pat.gnd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.kappich.pat.gnd.  If not, see <http://www.gnu.org/licenses/>.

 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436 
 * mail: <info@kappich.de>
 */

package de.kappich.pat.gnd.properties;

import de.bsvrz.sys.funclib.kappich.annotations.Nullable;
import de.kappich.pat.gnd.utils.SpringUtilities;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;
import java.awt.Dimension;
import java.awt.event.ItemListener;

/**
 * Diese Klasse verkapselt die Eigenschaft Stricheln.
 *
 * @author Kappich Systemberatung
 * @version $Revision$
 */
public class DottingProperty extends AbstractProperty {
	private static final DottingProperty _instance = new DottingProperty();
	private static final String KEY = "dotting";
	private static final DistanceRasterType DEFAULT_DOTTING = DistanceRasterType.FIFTY_THOUSAND;

	private DottingProperty() {
		super("Striche");
	}

	public static Property getInstance() {
		return _instance;
	}

	@Override
	public Object getDefaultValue() {
		return DEFAULT_DOTTING;
	}

	@Override
	public String getKey() {
		return KEY;
	}

	@Override
	public PropertyPanel getPropertyPanel(@Nullable Object value, boolean changeable) {
		if (value!=null) {
			return new DottingPanel((DistanceRasterType) value, changeable);
		} else {
			return new DottingPanel(DEFAULT_DOTTING, changeable);
		}
	}

	private static class DottingPanel extends JPanel implements PropertyPanel {
		private final JComboBox<Object> _dottingComboBox = new JComboBox<>(DistanceRasterType.values());

		public DottingPanel(DistanceRasterType distanceRasterType, boolean changeable) {
			JLabel label = new JLabel("Stricheln bis: ");
			_dottingComboBox.setMaximumSize(new Dimension(200, 25));
			_dottingComboBox.getModel().setSelectedItem(distanceRasterType);
			_dottingComboBox.setEnabled(changeable);
			setLayout(new SpringLayout());
			add(label);
			add(_dottingComboBox);
			SpringUtilities.makeCompactGrid(this, 2, 5, 5);
		}

		@Override
		public void addListener(final PropertyPanelListener listener) {
			ItemListener itemListener = e -> listener.stateChanged();
			_dottingComboBox.addItemListener(itemListener);
		}

		@Override
		public Object getPropertyValue() {
			return _dottingComboBox.getSelectedItem();
		}

		@Override
		public void setSelectionEnabled(final boolean enabled) {
			_dottingComboBox.setEnabled(enabled);
		}
	}
}
